/*                        GlintBox (Release 1.00)                         */
/*               MrFrosty (Hex Heroes) - 2nd November 1997                */
/*            website:  http://www.netyaroze-europe.com/~mrfrosty         */
/*              email:  j.t.rutherford@sms.ed.ac.uk                       */

/* Don't worry if you can't follow the code - it's a lot more complicated */
/*    than it has to be because I built it with the flexibility to add    */
/*    more later. Watch out for updates... Have fun!                      */


/* To run this effect, after having installed 'djsetup' type 'make'.      */
/*    When the code has compiled, enter 'siocons auto'.                   */

#include <libps.h>


//                   >>>  THINGS TO PLAY WITH  <<<

/* Uncomment DEBUG for VSYNC counter                */
/* Uncomment NTSC_SYSTEM for NTSC (default is PAL)  */

//#define DEBUG
//#define NTSC_SYSTEM

/* Alter the colour values to between 0 and 255                        */
/*    (rand()/128) produces a random value (0 to 255)                  */
/*    Setting one value at each end of the line produces a nice effect */

#define RED0   (rand()/128)
#define GREEN0 0
#define BLUE0  0
#define RED1   0
#define GREEN1 0
#define BLUE1  (rand()/128)


/* Play with the semitransparency effect               */
/*    <bit 30> Switches transparency on/off (1 is on)  */
/*    Bit 28 and 29 alter the transparency rate        */
/*       (See GsGLINE in the Library Reference)        */

#define LINE_ATTRIBUTE  ((1<<30)+(1<<28)+(1<<29))


/* MOVESPEED alters the speed of the points                  */
/*   Altering this will give a very wide variety of effects  */
/* Try:      MOVESPEED0                 MOVESPEED1           */
/*           ----------                 ----------           */
/*         (SCALEFACTOR*4)           (SCALEFACTOR*3)         */
/*          (SCALEFACTOR)            (SCALEFACTOR*2)         */
/*          (SCALEFACTOR)             (SCALEFACTOR)          */
/*          (SCALEFACTOR)           -(SCALEFACTOR*2)         */

#define MOVESPEED0   (SCALEFACTOR*4)
#define MOVESPEED1   (SCALEFACTOR*3)


//         >>>  THINGS YOU SHOULD BE CAREFUL PLAYING WITH  <<<

/* SCALEFACTOR sets the effect resolution */
#define SCALEFACTOR 32

#define OT_LENGTH    1

/* Ensure that the PACKETSIZE = (N_LINES * 25) plus a few bytes */
#define N_LINES 350
#define PACKETSIZE   8760


//                         >>> TYPEDEFS  <<<

typedef struct{
   short x0;
   short y0;
   short x1;
   short y1;
} BOX2D;

typedef struct{
   GsGLINE  gsgline;
   BOX2D*   box2d[2];
   short    xdir[2];
   short    ydir[2];
} BOX2DLINE;


//                    >>>  FUNCTION PROTOTYPES  <<<
                                
int   GlintBox(void);
void  SetupBOX2D(BOX2D* box2d, short x0, short y0, short x1, short y1);
void  SetupBOX2DLINE(BOX2DLINE* box2dline, u_long attribute, short x0, short y0, short x1, short y1, short r0, short g0, short b0, short r1, short g1, short b1, short xdir0, short ydir0, short xdir1, short ydir1, BOX2D* box2d0, BOX2D* box2d1);
void  MoveBOX2DLINE(BOX2DLINE* box2dline, short* lx, short* ly, int line);

//                         >>>  FUNCTIONS  <<<

void main(void)
{

   #ifdef NTSC_SYSTEM
      SetVideoMode(MODE_NTSC);
   #else
      SetVideoMode(MODE_PAL);
   #endif

   GsInitGraph(320, 240, 4, 0, 0);
   GsDefDispBuff(0, 0, 0, 240);
   FntLoad(960, 256);
   FntOpen(16, 16, 256, 200, 0, 512);

   GlintBox();
}

int GlintBox(void)
{
   GsOT     world_ot[2];
   GsOT_TAG world_ot_tag[2][1<<OT_LENGTH];

   PACKET   gpu_packet_area[2][PACKETSIZE];

   int   line;
   int   active_buffer;
   int   vcount;
   BOX2D       box2d[2];
   BOX2DLINE   box2dline[N_LINES];
   long boxperimeter0;
   long boxperimeter1;
   short xbox0;
   short ybox0;
   short xbox1;
   short ybox1;
   long boxperimeter0pos;
   long boxperimeter1pos;
   short xpos0;
   short ypos0;
   short xdir0;
   short ydir0;
   short xpos1;
   short ypos1;
   short xdir1;
   short ydir1;

   world_ot[0].length   = OT_LENGTH;
   world_ot[0].org      = world_ot_tag[0];
   world_ot[1].length   = OT_LENGTH;
   world_ot[1].org      = world_ot_tag[1];

   SetupBOX2D(&box2d[0], SCALEFACTOR*0, SCALEFACTOR*0, SCALEFACTOR*319, SCALEFACTOR*239);
   SetupBOX2D(&box2d[1], SCALEFACTOR*0, SCALEFACTOR*0, SCALEFACTOR*319, SCALEFACTOR*239);

   xbox0 = (box2d[0].x1 - box2d[0].x0);
   ybox0 = (box2d[0].y1 - box2d[0].y0);
   xbox1 = (box2d[1].x1 - box2d[1].x0);
   ybox1 = (box2d[1].y1 - box2d[1].y0);
   boxperimeter0 = (xbox0 + ybox0 + xbox0 + ybox0);
   boxperimeter1 = (2*(xbox1 + ybox1));

   for(line=0; line<N_LINES; line++)
   {
      boxperimeter0pos = (boxperimeter0 * line) / N_LINES;

      if(boxperimeter0pos <= xbox0)
      {
         xpos0 = box2d[0].x0 + boxperimeter0pos;
         ypos0 = box2d[0].y0;
         xdir0 = MOVESPEED0;
         ydir0 = 0;
      }
      else
      {
         if(boxperimeter0pos <= (xbox0 + ybox0))
         {
            xpos0 = box2d[0].x1;
            ypos0 = box2d[0].y0 + boxperimeter0pos - xbox0;
            xdir0 = 0;
            ydir0 = MOVESPEED0;
         }
         else
         {
            if(boxperimeter0pos <= (xbox0 + ybox0 + xbox0))
            {
               xpos0 = box2d[0].x1 - (boxperimeter0pos - xbox0 - ybox0);
               ypos0 = box2d[0].y1;
               xdir0 = -MOVESPEED0;
               ydir0 = 0;
            }
            else
            {
               if(boxperimeter0pos <= (xbox0 + ybox0 + xbox0 + ybox0))
               {
               xpos0 = box2d[0].x0;
               ypos0 = box2d[0].y1 - (boxperimeter0pos - xbox0 - ybox0 - xbox0);
               xdir0 = 0;
               ydir0 = -MOVESPEED0;
               }
               else
               {
                  printf("help!");
               }
            }
         }
      }


      boxperimeter1pos = (boxperimeter1 * line) / N_LINES;

      if(boxperimeter1pos <= xbox1)
      {
         xpos1 = box2d[1].x1 - boxperimeter1pos;
         ypos1 = box2d[1].y1;
         xdir1 = -MOVESPEED1;
         ydir1 = 0;
      }
      else
      {
         if(boxperimeter1pos <= (xbox1 + ybox1))
         {
            xpos1 = box2d[1].x0;
            ypos1 = box2d[1].y1 - (boxperimeter1pos - xbox1);
            xdir1 = 0;
            ydir1 = -MOVESPEED1;
         }
         else
         {
            if(boxperimeter1pos <= (xbox1 + ybox1 + xbox1))
            {
               xpos1 = box2d[1].x0 + (boxperimeter1pos - xbox1 - ybox1);
               ypos1 = box2d[1].y0;
               xdir1 = MOVESPEED1;
               ydir1 = 0;
            }
            else
            {
               if(boxperimeter1pos <= (xbox1 + ybox1 + xbox1 + ybox1))
               {
               xpos1 = box2d[1].x1;
               ypos1 = box2d[1].y0 + (boxperimeter1pos - xbox1 - ybox1 - xbox1);
               xdir1 = 0;
               ydir1 = MOVESPEED1;
               }
               else
               {
                  printf("help!");
               }
            }
         }
      }


      SetupBOX2DLINE(&box2dline[line], LINE_ATTRIBUTE, xpos0, ypos0, xpos1, ypos1, RED0, GREEN0, BLUE0, RED1, GREEN1, BLUE1, xdir0, ydir0, xdir1, ydir1, &box2d[0], &box2d[1]);
   }

   while(1)
   {
      active_buffer = GsGetActiveBuff();
      GsSetWorkBase(gpu_packet_area[active_buffer]);

      GsClearOt(0, 0, &world_ot[active_buffer]);

      for(line=0; line<N_LINES; line++)
      {
         MoveBOX2DLINE(&box2dline[line], &box2dline[line].gsgline.x0, &box2dline[line].gsgline.y0, 0);
         MoveBOX2DLINE(&box2dline[line], &box2dline[line].gsgline.x1, &box2dline[line].gsgline.y1, 1);
         GsSortGLine(&box2dline[line].gsgline, &world_ot[active_buffer], 0);
      }

      DrawSync(0);
      vcount = VSync(0);

      GsSwapDispBuff();


      GsSortClear(0, 0, 0, &world_ot[active_buffer]);

      GsDrawOt(&world_ot[active_buffer]);


#ifdef DEBUG
      FntPrint("vcount = %d\n", vcount);
#endif

      FntFlush(-1);
   };

   return(1);
}

void SetupBOX2D(BOX2D* box2d, short x0, short y0, short x1, short y1)
{
   box2d->x0 = x0;
   box2d->y0 = y0;
   box2d->x1 = x1;
   box2d->y1 = y1;
}


void SetupBOX2DLINE(BOX2DLINE* box2dline, u_long attribute, short x0, short y0, short x1, short y1, short r0, short g0, short b0, short r1, short g1, short b1, short xdir0, short ydir0, short xdir1, short ydir1, BOX2D* box2d0, BOX2D* box2d1)
{
   box2dline->gsgline.attribute = attribute;
   box2dline->gsgline.x0 = x0 / SCALEFACTOR;
   box2dline->gsgline.y0 = y0 / SCALEFACTOR;
   box2dline->gsgline.x1 = x1 / SCALEFACTOR;
   box2dline->gsgline.y1 = y1 / SCALEFACTOR;
   box2dline->gsgline.r0 = r0;
   box2dline->gsgline.g0 = g0;
   box2dline->gsgline.b0 = b0;
   box2dline->gsgline.r1 = r1;
   box2dline->gsgline.g1 = g1;
   box2dline->gsgline.b1 = b1;
   box2dline->xdir[0] = xdir0;
   box2dline->ydir[0] = ydir0;
   box2dline->xdir[1] = xdir1;
   box2dline->ydir[1] = ydir1;
   box2dline->box2d[0] = box2d0;
   box2dline->box2d[1] = box2d1;
}


void MoveBOX2DLINE(BOX2DLINE* box2dline, short* lx, short* ly, int line)
{
   short newx = ((*lx) * SCALEFACTOR) + box2dline->xdir[line];
   short newy = ((*ly) * SCALEFACTOR) + box2dline->ydir[line];

   if(newx > box2dline->box2d[line]->x1)
   {
      newy = box2dline->box2d[line]->y0 + (newx - box2dline->box2d[line]->x1);
      newx = box2dline->box2d[line]->x1;
      box2dline->ydir[line] = box2dline->xdir[line];
      box2dline->xdir[line] = 0;
   }

   if(newy > box2dline->box2d[line]->y1)
   {
      newx = box2dline->box2d[line]->x1 - (newy - box2dline->box2d[line]->y1);
      newy = box2dline->box2d[line]->y1;
      box2dline->xdir[line] = -box2dline->ydir[line];
      box2dline->ydir[line] = 0;
   }

   if(newx < box2dline->box2d[line]->x0)
   {
      newy = box2dline->box2d[line]->y1 - (box2dline->box2d[line]->x0 - newx);
      newx = box2dline->box2d[line]->x0;
      box2dline->ydir[line] = box2dline->xdir[line];
      box2dline->xdir[line] = 0;
   }

   if(newy < box2dline->box2d[line]->y0)
   {
      newx = box2dline->box2d[line]->x0 + (box2dline->box2d[line]->y0 - newy);
      newy = box2dline->box2d[line]->y0;
      box2dline->xdir[line] = -box2dline->ydir[line];
      box2dline->ydir[line] = 0;
   }
   
   (*lx) = (newx / SCALEFACTOR);
   (*ly) = (newy / SCALEFACTOR);

   //FntPrint("LX %d\nXS %d\n", *lx, (newx/SCALEFACTOR));
}
